#-----------------------------------------------------------------------------
#  Copyright (c) 2016 Eclektek LLC. All rights reserved.
#
#
#  Developed by: William Carter
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and  
#  associated documentation files (the "Software"), to deal with the Software without restriction,  
#  including without limitation the rights to use, copy, modify, merge, publish, sublicense and/or    
#  distribute copies of the Software, and to permit persons to whom the Software is furnished to do so,  
#  subject to the following conditions:
#
#      Redistributions of source code must retain the above copyright notice, this list of conditions 
#      and the following disclaimers.
#      Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
#      and the following disclaimers in the documentation and/or other materials provided with the distribution.
#      Neither the names of William Carter, Eclektek LLC, nor the names of its 
#      contributors may be used to endorse or promote products derived from this Software without 
#      specific prior written permission from Eclektek LLC.
#      This Software is provided free of charge and Eclektek LLC reserves the right to sell copies of the 
#      Software in it original form. Anyone sub-licensing this Software will provide it free of charge.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
#  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
#  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
#  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
#  SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
#-----------------------------------------------------------------------------
#  History :                                                                       
#                                                                                                                  
#  Date      By    Version  Comments                                         
#  --------  ----  -------  -----------------------------------------------------
#  12/09/16  W.C.  1.00     Created
#
#-----------------------------------------------------------------------------
# Test.py was written using Python 2.7 on Windows 7
# Test.py requires the following non-standard Library modules:
#   pyserial-2.6
#   
#-----------------------------------------------------------------------------
# Test.py communicates with the Differential PMOD CHallenge Testbed HDL component
# to setup, execute and acquire test results using a USB UART
#
# Test only functions with a Nexys Video or Genesys2 board when configured with one of the
# Differential PMOD Challenge toplevel builds.
#
# Command line Useage: <Python Path>python Test.py [-p port] [-l off]
#  where <Python Path> is the location of your Python version 2.7 or less installation
#  [port] is the name of the USB UART port connected to your FPGA board UART connector
#  [off] disables data logging ( anything else or absense of the -l options enables data logging
#
#   Windows example:  C:\Python27\python27 Test.py -p COM9 -l off
#   Linux example:    python Test.py -p /dev/ttyUSB0 
#
# Test first checks that the specified UART port is available. If not it exits. Test then
# tries to write a value to UART_RegAccess Register 0 and read it back. If unsuccessful it exits.
# If the UART_RegAccess test succeeds a menu is displayed in the terminal window or DOS box.
# Test Commands are:
#   'Q' - Quit Test
#   'N' - Set the Si570 to a new output frequency. Values are entered in Hz as integers ( no fractional part )
#   'M' - Display the current Si570 frequency measurement. Displayed in Mhz. 
#   'S' - Sweep test. You will be prompted to provide a lower and upper bound as well as an delta all in
#         Hz. Tests will be run at all frequencies from the lower through the upper bound in delta Hz increments.
#   'R' - Run a test at the current Si570 frequency
#   'E' - Enter EYE mode. Data source is run until a 'T' command ends it. This allows using the MONITOR_PMOD
#         data signals to evaluate the "eye" of each data bit ( delayed if the idelay option is enabled ) on
#         an oscilloscope
#   'Z' - Display registers R1 though R11
#   'C' - Clear all status registers
#   'T' - Force the Testbed state machine into the IDLE state and reset the Source and Sink DUT logic. 
#         Required to exit 'eye' mode. 
#   'D' - Set iDELAY values. You will be prompted to enter 4 values from 0 to 31 to set each of the iDELAY
#         delays ( only does anything if the if the idelay option is enabled in the FPGA build). 
#         Delays from 0 to ~ 2.5 ns are possible.
#   'X' - Displays the current FPGA Temperature sensor reading in Celsius degrees
#
# Test writes a uniquely timestamped log file, unless disabled by the '-l off' command line option, in the directory
# where Test.py is located. A new file is created each time that Test is run.
#
# Typically, you might want to run the 'S' command to run through a set of frequencies. Be aware that it might
# be that successful runs occur at particular frequencies related to your sampling clock frequency. It is likely
# that the highest Si570 frequency that will produce a successful test run is found by using the 'N' command to
# set a particular frequency and then a 'R' command to perform a test run at that frequency. Of course, if you want
# there's nothing stopping you from running a Sweep test from  10.5 - 100 MHz in 250 KHz increments.
#  
# A successful test will end with R4 = 0x000000100000 = 1048576 words written to the testbed by the Data Sink without
# an error. R5 displays the total elapsed time that the testbed state machine was in the RUN state and is the time from when
# the first word was sent to the Data Source to when the 1048576th word was received from the Data Sink. The
# elapsed time in seconds is R5/100,000,000. The 32-bit Word data rate is 1048576/elapsed time = 1048576*100000000/R5 
#
# The 'X' command works even if there is nothing connected to the FMC connector. You can verify the fan control by
# configuring the Genesys2, entering "eye" mode, waiting for the fan to come on and executing the 'X' command as soon 
# as the fan comes on and until it goes off again.
# 
#-----------------------------------------------------------------------------
# NOTES:
#
#   - in this version of Test not all status register fields are extracted and displayed
#   - on some FPGA boards the UART is powered by the USB hub or PC and will be enumerated and available whenever a
#     cable connects the FPGA UART connector to a PC or PC connected hub, even if the FPGA board power switch is off.
#   - in Windows 7 the appropriate UART port can be found using the Windows Device Manager and is typically COMxx. In
#     Linux typing 'dmesg | grep tty' in a terminal will display all of the USB devices that have been enumerated. In either OS
#     comparing what's available when the FPGA UART is not attached to what's available when it is attached helps 
#     identify the correct port in case of confusion.
#   - the lowest output frequency supported by the Si570 on the Xilinx HW-FMC-105-DEBUG board is 10 MHz, though the
#     lowest I've been able to set is around 10.5 MHZ. See the Si570 datasheet from Silicon Labs for information.
# 
#-----------------------------------------------------------------------------
# ToDo:
#
#
#-----------------------------------------------------------------------------
# KNOWN ISSUES:
#
#
#-----------------------------------------------------------------------------

# Modules
import serial
import time
import math
import sys

# Some definitions

# FPGA UART_RegAccess Register Assignments
test_control_reg          = '0'
test_rxfifo_data_reg      = '2'
test_fifo_fill_count_reg  = '3'
test_status_reg           = '4'
channel_transcnt_reg      = '5'
link_reset_count_reg      = '6'
test_readback_reg         = '8'
xadc_write_reg            = 'A'
xadc_read_reg             = 'B'

# Control and Status bit field assignments
## Reg0
sys_reset_mask            = 0x800000000000
gt_reset_mask             = 0x400000000000
quality_counter_rst_mask  = 0x200000000000
test_fsm_rst_mask         = 0x100000000000
test_enable_rst_mask      = 0x080000000000
test_fifo_reset_mask      = 0x000000001000
counter_sel_mask          = 0x000000000180
counter_sel_mask          = 0x000000000070
cdrovrden_mask            = 0x000000000008
loopback                  = 0x000000000007

## Reg4
hard_error_mask           = 0x800000000000
soft_error_mask           = 0x400000000000
reset_test_failed_mask    = 0x000000008000
reset_test_passed_mask    = 0x000000004000
test_fifo_full_mask       = 0x000000002000
channel_up_mask           = 0x000000001000
lane_up_mask              = 0x000000000F00
data_error_count_mask     = 0x0000000000FF

## Reg10
xadc_wr_bit_mask          = 0x800000000000
xadc_address_mask         = 0x000001FF0000
xadc_write_data           = 0x00000000FFFF
## Reg11
xadc_temp_ot_alarm_mask   = 0x800000000000
xadc_alarm_mask           = 0x7F0000000000
xadc_read_data            = 0x00000000FFFF

user_clock_freq = 42187500.0

# command line options
uart_port = 'COM9'

# Functions

def get_results():
  t = [0,0]
  TestData = 'R 4\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  x = int(ResponseData[4:18],16)
  t[0] = x
  TestData = 'R 5\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  x = int(ResponseData[4:18],16)
  t[1] = x
  return t
  
      
# get command-line options
def get_options(argv):
  clo= {}
  while argv:
    if argv[0][0] == '-':
      clo[argv[0]] = argv[1]
      argv = argv[2:]
    else:
      argv = argv[1:]
  return clo
  
 
# Display FPGA core temperature in Celsius degrees
def get_xadc_temp():
  TestData = 'W ' + xadc_write_reg + ' 0' + '\x0D\x0A'
  ser.write(TestData)
  time.sleep(.2)              # delay 
  TestData = 'R ' + xadc_read_reg + '\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  n = int(ResponseData[4:18], 16)
  t = float(n)*2015.0/(16.0*16384.0) - 273.0
  print 'FPGA Core Temperature = ', t   
 
# Display the net payload transfer rate
def get_xfer_rate():
  # reset the data fifo and clear the time counter
  TestData = 'W ' + test_control_reg + ' 1000' + '\x0D\x0A'
  ser.write(TestData)
  # enable data fifo writes and allow time counter to run
  TestData = 'W ' + test_control_reg + ' 0' + '\x0D\x0A'
  ser.write(TestData)
  # wait for the fifo to fill
  time.sleep(.5)
  # read the elapsed time count       
  TestData = 'R ' + test_fifo_fill_count_reg + ' \x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  n = int(ResponseData[4:18], 16)
  t = (float(n)/user_clock_freq)
  r = 32*131072.0/(t*1000000.0)
  print 'Aurora Data transfer Rate (MB/s) = ', r  
  

# function to read all status registers
def read_status():
  TestData = 'R 2\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg2 = ', ResponseData[4:18]
  TestData = 'R 3\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg3 = ', ResponseData[4:18]
  TestData = 'R 4\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg4 = ', ResponseData[4:18]
  TestData = 'R 5\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg5 = ', ResponseData[4:18]
  TestData = 'R 6\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg6 = ', ResponseData[4:18]
  TestData = 'R 8\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg8 = ', ResponseData[4:18]
  TestData = 'R B\x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reg11 = ', ResponseData[4:18]
  
  
def print_cmds():
  print '\nCommands: Q D Z X'  
  
 
# Begin by checking for command line options
#   -p 'USB PORT NAME'  - USB PORT NAME examples 'COM11' , 'USB1'
#   -l  'off'           - 'off' disable writing log file, anything else is enable (default) 
cmd_line_options = get_options(sys.argv)
if cmd_line_options.has_key('-p') == 1:
  uart_port = cmd_line_options['-p']
  uart_port = uart_port
  print 'Using UART PORT: ' + uart_port
      
  

# Start by trying to open the specified UART port
try:
  ser = serial.Serial(port=uart_port)
except:  
  uart_port_is_open = False
  print uart_port + ' is not available'
else:
  uart_port_is_open = True
  ser.baudrate = 921600 
  # select FIVEBITS, SIXBITS, SEVENBITS or EIGHTBITS of data
  #ser.bytesize = serial.FIVEBITS
  #ser.bytesize = serial.SIXBITS
  #ser.bytesize = serial.SEVENBITS
  ser.bytesize = serial.EIGHTBITS
  
  # select STOPBITS_ONE, STOPBIT_TWO or STOPBITS_ONE_POINT_FIVE
  ser.stopbits = serial.STOPBITS_ONE
  #ser.stopbits = serial.STOPBIT_TWO
  #ser.stopbits = serial.STOPBITS_ONE_POINT_FIVE
  
  # select PARITY_NONE, PARITY_EVEN or PARITY_ODD
  ser.parity = serial.PARITY_NONE
  #ser.parity = serial.PARITY_EVEN
  #ser.parity = serial.PARITY_ODD
  
  ser.xonxoff = 0
  ser.rtscts = 0
  # IF you don't set the timeout for reads the code will hang
  ser.timeout = 0.5
  
  #ser.parity = serial.PARITY_EVEN
  
# only continue if the specified UART connection exists  
if uart_port_is_open == True:
    
  # Test R0 to see if we can communicate with the FPGA
  print 'Writing R8 with 0x112233445566 to test interface'
  TestData = 'W ' + test_readback_reg + ' 112233445566\x0D\x0A' # write command followed by CRLF
  ser.write(TestData)
  TestData = 'R ' + test_readback_reg + ' \x0D\x0A'
  ser.write(TestData)
  ResponseData = ser.read(19)
  print 'Reading R8: ',ResponseData
  if ResponseData[:18] == '0x8 0x112233445566':
    print 'Register write/read Test Successful'
    has_connection = True
  else:  
    print 'Register write/read Test Failed!'
    has_connection = False
  if has_connection == True:
    
    u = '~'
    while 1:
      print_cmds()
      u = raw_input('\nEnter Command => ')
      # Quit the test
      if u.upper() == 'Q':
        break
      
      # Run a data rate test registers    
      elif u.upper() == 'D':
        get_xfer_rate()
      
      # Print Control/Status registers    
      elif u.upper() == 'Z':
        read_status()
      
      # Display the FPGA core temperature        
      elif u.upper() == 'X':
        get_xadc_temp()
        
      else:
        print '\nNot a valid Command'  
  ser.close

